import { coerceBooleanProperty } from '@angular/cdk/coercion';
import { Attribute, Component, EventEmitter, Input, Optional, Output, Self, ViewEncapsulation, isDevMode, ContentChildren, NgZone, ChangeDetectorRef, ChangeDetectionStrategy, NgModule } from '@angular/core';
import { NgControl } from '@angular/forms';
import { NtFormFieldControl } from '@ng-tangram/components/forms';
import { defer } from 'rxjs/observable/defer';
import { merge } from 'rxjs/observable/merge';
import { switchMap, take, takeUntil, startWith } from 'rxjs/operators';
import { Subject } from 'rxjs/Subject';
import { CommonModule } from '@angular/common';

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
var /** @type {?} */ uniqueId = 0;
/**
 * @template T
 */
var  /**
 * @template T
 */
NtCheckboxChange = /** @class */ (function () {
    function NtCheckboxChange(source, checked) {
        this.source = source;
        this.checked = checked;
    }
    return NtCheckboxChange;
}());
/**
 * @template T
 */
var NtCheckboxComponent = /** @class */ (function () {
    function NtCheckboxComponent(tabIndex, ngControl) {
        this.ngControl = ngControl;
        this.id = "nt-checkbox-" + uniqueId++;
        this._disabled = false;
        this._readonly = false;
        this._checked = false;
        this.change = new EventEmitter();
        this._onChange = function () { };
        this._onTouched = function () { };
        if (this.ngControl) {
            this.ngControl.valueAccessor = this;
        }
        this.tabIndex = parseInt(tabIndex) || 0;
    }
    Object.defineProperty(NtCheckboxComponent.prototype, "value", {
        get: /**
         * @return {?}
         */
        function () { return this._value; },
        set: /**
         * @param {?} value
         * @return {?}
         */
        function (value) { this._value = value; },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(NtCheckboxComponent.prototype, "disabled", {
        get: /**
         * @return {?}
         */
        function () { return this._disabled; },
        set: /**
         * @param {?} value
         * @return {?}
         */
        function (value) { this._disabled = coerceBooleanProperty(value); },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(NtCheckboxComponent.prototype, "readonly", {
        get: /**
         * @return {?}
         */
        function () { return this._readonly; },
        set: /**
         * @param {?} value
         * @return {?}
         */
        function (value) { this._readonly = coerceBooleanProperty(value); },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(NtCheckboxComponent.prototype, "checked", {
        get: /**
         * @return {?}
         */
        function () { return this._checked; },
        set: /**
         * @param {?} value
         * @return {?}
         */
        function (value) { this._checked = coerceBooleanProperty(value); },
        enumerable: true,
        configurable: true
    });
    /**
     * @param {?} value
     * @return {?}
     */
    NtCheckboxComponent.prototype.writeValue = /**
     * @param {?} value
     * @return {?}
     */
    function (value) {
        this._checked = !!value;
    };
    /**
     * @param {?} fn
     * @return {?}
     */
    NtCheckboxComponent.prototype.registerOnChange = /**
     * @param {?} fn
     * @return {?}
     */
    function (fn) {
        this._onChange = fn;
    };
    /**
     * @param {?} fn
     * @return {?}
     */
    NtCheckboxComponent.prototype.registerOnTouched = /**
     * @param {?} fn
     * @return {?}
     */
    function (fn) {
        this._onTouched = fn;
    };
    /**
     * @param {?} isDisabled
     * @return {?}
     */
    NtCheckboxComponent.prototype.setDisabledState = /**
     * @param {?} isDisabled
     * @return {?}
     */
    function (isDisabled) {
        this.disabled = isDisabled;
    };
    /**
     * @param {?} event
     * @return {?}
     */
    NtCheckboxComponent.prototype._onInputClick = /**
     * @param {?} event
     * @return {?}
     */
    function (event) {
        event.stopPropagation();
        if (!this.disabled) {
            this._checked = !this._checked;
            this._onTouched();
            this._emitChangeEvent();
        }
    };
    /**
     * @return {?}
     */
    NtCheckboxComponent.prototype._emitChangeEvent = /**
     * @return {?}
     */
    function () {
        var /** @type {?} */ event = new NtCheckboxChange(this, this.checked);
        this._onChange(this.checked);
        this.change.emit(event);
    };
    NtCheckboxComponent.decorators = [
        { type: Component, args: [{
                    selector: 'nt-checkbox',
                    encapsulation: ViewEncapsulation.None,
                    template: "<label class=\"nt-checkbox-layout\"> <div class=\"nt-checkbox-input-container\"> <input class=\"cdk-visually-hidden\" type=\"checkbox\" [tabIndex]=\"tabIndex\" [checked]=\"checked\" [disabled]=\"disabled\" (click)=\"_onInputClick($event)\"> <div class=\"nt-checkbox-input\"></div> </div> <span class=\"nt-checkbox-label\"> <ng-content></ng-content> </span> </label> ",
                    providers: [
                        { provide: NtFormFieldControl, useExisting: NtCheckboxComponent }
                    ],
                    host: {
                        'class': 'nt-checkbox',
                        '[class.disabled]': 'disabled'
                    }
                },] },
    ];
    /** @nocollapse */
    NtCheckboxComponent.ctorParameters = function () { return [
        { type: undefined, decorators: [{ type: Attribute, args: ['tabindex',] },] },
        { type: NgControl, decorators: [{ type: Self }, { type: Optional },] },
    ]; };
    NtCheckboxComponent.propDecorators = {
        "value": [{ type: Input },],
        "disabled": [{ type: Input },],
        "readonly": [{ type: Input },],
        "checked": [{ type: Input },],
        "change": [{ type: Output },],
    };
    return NtCheckboxComponent;
}());

/*! *****************************************************************************
Copyright (c) Microsoft Corporation. All rights reserved.
Licensed under the Apache License, Version 2.0 (the "License"); you may not use
this file except in compliance with the License. You may obtain a copy of the
License at http://www.apache.org/licenses/LICENSE-2.0

THIS CODE IS PROVIDED ON AN *AS IS* BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
KIND, EITHER EXPRESS OR IMPLIED, INCLUDING WITHOUT LIMITATION ANY IMPLIED
WARRANTIES OR CONDITIONS OF TITLE, FITNESS FOR A PARTICULAR PURPOSE,
MERCHANTABLITY OR NON-INFRINGEMENT.

See the Apache Version 2.0 License for specific language governing permissions
and limitations under the License.
***************************************************************************** */
/* global Reflect, Promise */

var extendStatics = Object.setPrototypeOf ||
    ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
    function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };

function __extends(d, b) {
    extendStatics(d, b);
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
/**
 * @return {?}
 */
function getNtCheckboxGroupNonFunctionValueError() {
    return Error('`compareWith` must be a function.');
}
var /** @type {?} */ uniqueId$1 = 0;
/**
 * @template T
 */
var NtCheckboxGroupComponent = /** @class */ (function (_super) {
    __extends(NtCheckboxGroupComponent, _super);
    function NtCheckboxGroupComponent(_ngZone, _changeDetectorRef, ngControl) {
        var _this = _super.call(this) || this;
        _this._ngZone = _ngZone;
        _this._changeDetectorRef = _changeDetectorRef;
        _this.ngControl = ngControl;
        _this.id = "nt-checkbox-group-" + uniqueId$1++;
        _this._destroy = new Subject();
        _this._disabled = false;
        _this._readonly = false;
        _this._required = false;
        _this._compareWith = function (o1, o2) { return o1 === o2; };
        _this.checkedChanges = defer(function () {
            if (_this.checkboxes) {
                return merge.apply(void 0, _this.checkboxes.map(function (item) { return item.change; }));
            }
            return _this._ngZone.onStable
                .asObservable()
                .pipe(take(1), switchMap(function () { return _this.checkedChanges; }));
        });
        /**
         * Emits when the value changes (either due to user input or programmatic change).
         */
        _this._onChange = function () { };
        _this._onTouched = function () { };
        if (_this.ngControl) {
            _this.ngControl.valueAccessor = _this;
        }
        return _this;
    }
    Object.defineProperty(NtCheckboxGroupComponent.prototype, "value", {
        get: /**
         * @return {?}
         */
        function () { return this._value; },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(NtCheckboxGroupComponent.prototype, "disabled", {
        get: /**
         * @return {?}
         */
        function () { return this._disabled; },
        set: /**
         * @param {?} value
         * @return {?}
         */
        function (value) { this._disabled = coerceBooleanProperty(value); },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(NtCheckboxGroupComponent.prototype, "required", {
        get: /**
         * @return {?}
         */
        function () { return this._required; },
        set: /**
         * @param {?} value
         * @return {?}
         */
        function (value) { this._required = coerceBooleanProperty(value); },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(NtCheckboxGroupComponent.prototype, "readonly", {
        get: /**
         * @return {?}
         */
        function () { return this._readonly; },
        set: /**
         * @param {?} value
         * @return {?}
         */
        function (value) { this._readonly = coerceBooleanProperty(value); },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(NtCheckboxGroupComponent.prototype, "compareWith", {
        set: /**
         * @param {?} fn
         * @return {?}
         */
        function (fn) {
            if (typeof fn !== 'function') {
                throw getNtCheckboxGroupNonFunctionValueError();
            }
            else {
                this._compareWith = fn;
            }
        },
        enumerable: true,
        configurable: true
    });
    /**
     * @return {?}
     */
    NtCheckboxGroupComponent.prototype.ngAfterViewInit = /**
     * @return {?}
     */
    function () {
        var _this = this;
        this.checkboxes.changes.pipe(startWith(null), takeUntil(this._destroy)).subscribe(function () {
            _this._initializeChecked();
        });
        this.checkedChanges.pipe(takeUntil(this._destroy)).subscribe(function () {
            _this._setValues();
        });
    };
    /**
     * @return {?}
     */
    NtCheckboxGroupComponent.prototype.ngOnDestroy = /**
     * @return {?}
     */
    function () {
        this._destroy.next();
        this._destroy.complete();
    };
    /**
     * @param {?} value
     * @return {?}
     */
    NtCheckboxGroupComponent.prototype.writeValue = /**
     * @param {?} value
     * @return {?}
     */
    function (value) {
        if (this.checkboxes) {
            this._setCheckedByValue(value);
        }
    };
    /**
     * @param {?} fn
     * @return {?}
     */
    NtCheckboxGroupComponent.prototype.registerOnChange = /**
     * @param {?} fn
     * @return {?}
     */
    function (fn) {
        this._onChange = fn;
    };
    /**
     * @param {?} fn
     * @return {?}
     */
    NtCheckboxGroupComponent.prototype.registerOnTouched = /**
     * @param {?} fn
     * @return {?}
     */
    function (fn) {
        this._onTouched = fn;
    };
    /**
     * @return {?}
     */
    NtCheckboxGroupComponent.prototype.focus = /**
     * @return {?}
     */
    function () {
        if (!this.disabled) {
            this._onTouched();
        }
    };
    /**
     * @return {?}
     */
    NtCheckboxGroupComponent.prototype._initializeChecked = /**
     * @return {?}
     */
    function () {
        var _this = this;
        Promise.resolve().then(function () {
            _this._setCheckedByValue(_this.ngControl ? _this.ngControl.value : _this.value);
        });
    };
    /**
     * @param {?} value
     * @return {?}
     */
    NtCheckboxGroupComponent.prototype._setCheckedByValue = /**
     * @param {?} value
     * @return {?}
     */
    function (value) {
        var _this = this;
        if (value) {
            this._value = value;
            this.checkboxes.forEach(function (item) { return item.checked = false; });
            value.forEach(function (val) { return _this._checkedValue(val); });
            this._changeDetectorRef.markForCheck();
        }
    };
    /**
     * @param {?} value
     * @return {?}
     */
    NtCheckboxGroupComponent.prototype._checkedValue = /**
     * @param {?} value
     * @return {?}
     */
    function (value) {
        var _this = this;
        var /** @type {?} */ correspondingItem = this.checkboxes.find(function (item) {
            try {
                return item.value != null && _this._compareWith(item.value, value);
            }
            catch (/** @type {?} */ error) {
                if (isDevMode()) {
                    console.warn(error);
                }
                return false;
            }
        });
        if (correspondingItem) {
            correspondingItem.checked = true;
        }
        return correspondingItem;
    };
    /**
     * @return {?}
     */
    NtCheckboxGroupComponent.prototype._setValues = /**
     * @return {?}
     */
    function () {
        var _this = this;
        this._value = [];
        this.checkboxes.forEach(function (item) {
            if (item.checked && !!item.value) {
                _this._value.push(item.value);
            }
        });
        this._onChange(this._value);
        this._onTouched();
        this._changeDetectorRef.markForCheck();
    };
    NtCheckboxGroupComponent.decorators = [
        { type: Component, args: [{
                    selector: 'nt-checkbox-group',
                    template: '<ng-content></ng-content>',
                    encapsulation: ViewEncapsulation.None,
                    changeDetection: ChangeDetectionStrategy.OnPush,
                    providers: [
                        { provide: NtFormFieldControl, useExisting: NtCheckboxGroupComponent }
                    ],
                    host: {
                        class: 'nt-checkbox-group nt-form-control'
                    }
                },] },
    ];
    /** @nocollapse */
    NtCheckboxGroupComponent.ctorParameters = function () { return [
        { type: NgZone, },
        { type: ChangeDetectorRef, },
        { type: NgControl, decorators: [{ type: Self }, { type: Optional },] },
    ]; };
    NtCheckboxGroupComponent.propDecorators = {
        "disabled": [{ type: Input },],
        "required": [{ type: Input },],
        "readonly": [{ type: Input },],
        "checkboxes": [{ type: ContentChildren, args: [NtCheckboxComponent,] },],
        "compareWith": [{ type: Input },],
    };
    return NtCheckboxGroupComponent;
}(NtFormFieldControl));

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
var NtCheckboxModule = /** @class */ (function () {
    function NtCheckboxModule() {
    }
    NtCheckboxModule.decorators = [
        { type: NgModule, args: [{
                    imports: [CommonModule],
                    exports: [NtCheckboxComponent, NtCheckboxGroupComponent],
                    declarations: [NtCheckboxComponent, NtCheckboxGroupComponent],
                },] },
    ];
    return NtCheckboxModule;
}());

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */

export { NtCheckboxChange, NtCheckboxComponent, getNtCheckboxGroupNonFunctionValueError, NtCheckboxGroupComponent, NtCheckboxModule };
//# sourceMappingURL=checkbox.js.map
